//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
float4 g_MaterialAmbient;		// Material's ambient color
float4 g_MaterialDiffuse;		// Material's diffuse color
float4 g_MaterialSpecular;		// Material's specular color
float g_Shininess;				// Material's shininess factor

// This effect file uses a single directional light (for now)
float4 g_LightDir;				// Light's direction
float4 g_LightColor;			// Light's color

texture g_MeshTexture;			// Color texture for mesh
texture g_EnvironmentTexture;	// cube map

float4 g_EyePoint;				// CameraPoint
float4x4 g_mWorld;				// World matrix for object
float4x4 g_mWorldInvTrans;		// Inverse transpose of world matrix
float4x4 g_mViewProjection;		// View * Projection matrix
float4x4 g_mInvWorldViewProjection;		// inverse world view projection matrix

bool g_UseTexture;				// flag for usage of texture for current pass
float g_time;					// used for animate

//-----------------------------------------------------------------------------
// Texture samplers
//-----------------------------------------------------------------------------
sampler MeshTextureSampler = 
sampler_state
{
    texture = <g_MeshTexture>;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
    MIPFILTER = LINEAR;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

sampler EnvironmentSampler = sampler_state
{ 
    Texture = (g_EnvironmentTexture);
    MipFilter = POINT;
    MinFilter = POINT;
    MagFilter = POINT;
};

//-----------------------------------------------------------------------------
// Vertex shader output structure
//-----------------------------------------------------------------------------
struct VS_OUTPUT
{
    float4 Position   		: POSITION;   	// vertex position 
    float2 TextureUV  		: TEXCOORD0;  	// vertex texture coords 
    float3 WorldNormal	    : TEXCOORD1;  	// vertex normal in world space
    float3 WorldPos			: TEXCOORD2;	// vertex position in world space
};


//-----------------------------------------------------------------------------
// Name: RenderSceneVS
// Type: Vertex shader                                      
// Desc: This shader computes standard transform
//-----------------------------------------------------------------------------
VS_OUTPUT RenderSceneVS( float4 vPos : POSITION, 
                         float4 vNormal : NORMAL,
                         float2 vTexCoord0 : TEXCOORD0 )
{
    VS_OUTPUT Out; 
    // Transform the position from object space to homogeneous projection space
    float4x4 worldViewProj = mul(g_mWorld,g_mViewProjection);
    Out.Position = mul(vPos, worldViewProj);
    Out.WorldNormal = normalize(mul(vNormal, g_mWorldInvTrans).xyz); //normal vector in world space
    Out.WorldPos = mul(vPos, g_mWorld).xyz; // vertex position in world space
    // Just copy the texture coordinate through
    Out.TextureUV = vTexCoord0; 
    
    return Out;    
}

//-----------------------------------------------------------------------------
// Pixel shader output structure
//-----------------------------------------------------------------------------
struct PS_OUTPUT
{
    float4 RGBColor : COLOR0;  // Pixel color    
};

//-----------------------------------------------------------------------------
// Name: RenderSceneWithTexturePS                                        
// Type: Pixel shader
// Desc: This shader computes phong lightning for finding pixel color 
//-----------------------------------------------------------------------------
PS_OUTPUT RenderScenePS( VS_OUTPUT In ) 
{ 
    PS_OUTPUT Output;
    float3 N = normalize(In.WorldNormal);
	float3 E = normalize(g_EyePoint.xyz - In.WorldPos); //eye vector
    float3 L = normalize( -g_LightDir.xyz); //light vector
    float3 H = normalize(E + L); //half angle vector
    //calculate the diffuse and specular contributions
    float diff = dot(N,L);
    if (diff < 0) diff = 0;
    float  spec = pow( max(0 , dot(N,H) ) , g_Shininess );
    // ambient + diffuse color
    float4 diffAmbColor = g_MaterialAmbient * g_LightColor + g_MaterialDiffuse * g_LightColor * diff;
    // specular color
    float4 specColor = g_MaterialSpecular * g_LightColor * spec;
    
	float4 diffuseTexture = g_UseTexture ? tex2D(MeshTextureSampler, In.TextureUV) : float4(1.0f,1.0f,1.0f,1.0f);
  	
  	Output.RGBColor = diffAmbColor * diffuseTexture + specColor;
    return Output;
}

//-----------------------------------------------------------------------------
// Skybox stuff
//-----------------------------------------------------------------------------
struct SkyboxVS_Input
{
    float4 Pos : POSITION;
};

struct SkyboxVS_Output
{
    float4 Pos : POSITION;
    float3 Tex : TEXCOORD0;
};

SkyboxVS_Output SkyboxVS( SkyboxVS_Input Input )
{
    SkyboxVS_Output Output;
    
    Output.Pos = Input.Pos;
    Output.Tex = normalize( mul(Input.Pos, g_mInvWorldViewProjection) );
    
    return Output;
}

float4 SkyboxPS( SkyboxVS_Output Input ) : COLOR
{
    float4 color = texCUBE( EnvironmentSampler, Input.Tex );
    return color;
}


//-----------------------------------------------------------------------------
// Techniques for rendering
//-----------------------------------------------------------------------------
technique RenderScene
{
    pass P0
    {          
        VertexShader = compile vs_2_0 RenderSceneVS();
        PixelShader  = compile ps_2_0 RenderScenePS();
    }
}

technique Skybox
{
    pass p0
    {
        VertexShader = compile vs_2_0 SkyboxVS();
        PixelShader = compile ps_2_0 SkyboxPS();
    }
}